---------------------------------------------------
---------------------------------------------------
--  Easy 1 Processor Architecture				 --
--  University of Puerto Rico					 --
--  Mayaguez Campus								 --
--	Department of Computer Engineering			 --
-- 												 --
--	Copyright (c) 2003							 --
--	All rigths reserved.						 --
--										   		 --
--	Prof. Bienvenido Vlez						 --
--	Abdiel Avils								 --
--	Rafael S. Garca							 --
--	Nesmary Hernndez 							 --
-- 	Carlos Prez								 --
---------------------------------------------------
---------------------------------------------------
-- 	[easy1_alu.vhd]
--	Easy 1 Arithmetic & Logic Unit
--	This module describes the Easy 1 ALU.
--	The Easy 1 ALU is composed of a multiplexer
--	connected to the Accumulator register. The
--	multiplexer receives the ouput from various
--	modules that perfom the logical and arithmetical
--	operations on the operands.
---------------------------------------------------	
---------------------------------------------------
--	Libraries declarations
library ieee;
use ieee.std_logic_1164.all;   
use easy1_package.all;
--
---------------------------------------------------
--	Entity declaration
--	@a0        	- operand from the A Bus
--	@b0        	- operand from the Accumulator
--	@alu_op		- signal to select the mux output
--	@ac_alu   	- output from the ALU to the Accumulator
--					 
entity easy1_alu is
	
	port (
		a0      : in  std_logic_vector(15 downto 0):=X"0000";
		b0      : in  std_logic_vector(15 downto 0):=X"0000";
		alu_op  : in  std_logic_vector( 2 downto 0):="000";
		ac_alu  : out std_logic_vector(15 downto 0):=X"0000");
	
end easy1_alu;
--
----------------------------------------------------
--	Architecture declaration
--	Signals:
--		@shift_op  		- signal that holds the shift
--						  module operand
--		@add_op    		- signal that holds the adder
--						  module operand
--		@add_cout  		- signal that holds the adder
--						  module carry out
--
--	Components :
--		@easy1_add   	- adder module
--		@easy1_shift 	- shifter module
--
architecture arq of easy1_alu is

	component easy1_add    
		port (
			a    : in  std_logic_vector(15 downto 0):=X"0000";
			b    : in  std_logic_vector(15 downto 0):=X"0000";
			sum  : out std_logic_vector(15 downto 0):=X"0000";
			cout : out std_logic:='0');
	end component;					   
	
	component easy1_shift
		port (  
			f     : in  std_logic_vector(15 downto 0):=X"0000";
			result: out std_logic_vector(15 downto 0):=X"0000");
	end component;
	
	signal shift_op : std_logic_vector(15 downto 0):=X"0000";
	signal add_op   : std_logic_vector(15 downto 0):=X"0000"; 
	signal add_cout : std_logic:='0';

begin 

	--	Port mapping:
	--		Component instances are mapped to internal signals,
	--		inputs, and outputs.
	adder : easy1_add 
	port map (
		a   => a0,
		b   => b0,
		sum => add_op,
		cout=> add_cout);
		
	shifter : easy1_shift
	port map (
		f      => b0,
		result => shift_op);

	--	Sensitivity List :
	--		This list holds the signals that modify and 
	--		control the multiplexer and the modules.
	process (alu_op, a0, b0, add_op, shift_op)  
	begin	  
		--	Multiplexer
		case alu_op is 
			--	Pass the data from the A Bus directly
			--	to the Accumulator.
			when "000" =>
			ac_alu<= a0;
			--	Perform the NOT operation on   
			--	b0 operand.
			when "001" =>
			ac_alu<= not b0 after DELAY_1_LEVEL;
			--	Perform the AND operation on a0  
			--	operand with b0 operand.
			when "010" =>
			ac_alu<=a0 and b0 after DELAY_1_LEVEL;
			--	Perform the ADD operation on a0  
			--	operand with b0 operand.
			when "011" =>
			ac_alu<= add_op after DELAY_5_LEVEL;
			--	Perform the Shift operation on   
			--	b0 operand.
			when "100"=>
			ac_alu<= shift_op;
			when others => null;
		end case;
	end process;
end arq;
----------------------------------------------------
----------------------------------------------------